//
//  OpenGL Triangle widget
//
#include "triangle.h"
#include <QtOpenGL>
#include <QMessageBox>

//
//  Constructor
//
Triangle::Triangle(QWidget* parent)
    : QGLWidget(parent)
{
   //  Set intial display angle
   theta = 0;
   //  Initialize color/texture/shader mode
   mode = 0;
   //  Start timing
   time.start();
   //  Start 100 fps timer connected to updateGL
   timer.setInterval(10);
   connect(&timer,SIGNAL(timeout()),this,SLOT(updateGL()));
   timer.start();
}

//
//  Set theta (rotation angle)
//
void Triangle::setTheta(int angle)
{
   //  Set angle
   theta = angle;
   //  Request redisplay
   updateGL();
}

//
//  Set color or texture or shader
//
void Triangle::setMode(QString type)
{
   if (type.contains(tr("Texture")))
      mode = 1;
   else if (type.contains(tr("Shader")))
      mode = 2;
   else
      mode = 0;
   //  Request redisplay
   updateGL();
}

//
//  Initialize
//
void Triangle::initializeGL()
{
   //  Enable Z-buffer depth testing
   glEnable(GL_DEPTH_TEST);
   //  Load Darth texture
   QPixmap img("darth.jpg");
   darth = bindTexture(img,GL_TEXTURE_2D);
   //  Build shader
   if (!shader.addShaderFromSourceFile(QGLShader::Vertex,"qthw.vert"))
      Fatal("Error compiling qthw.vert\n"+shader.log());
   if (!shader.addShaderFromSourceFile(QGLShader::Fragment,"qthw.frag"))
      Fatal("Error compiling qthw.frag\n"+shader.log());
   if (!shader.link())
      Fatal("Error linking shader\n"+shader.log());
}

//
//  Set projection when window is resized
//
void Triangle::resizeGL(int width, int height)
{
   //  Window aspect ration
   float asp= height ? width / (float)height : 1;

   //  Viewport is whole screen
   glViewport(0,0,width,height);

   //  Orthogonal projection to 1.5
   glMatrixMode(GL_PROJECTION);
   glLoadIdentity();
   glOrtho(-1.5*asp, +1.5*asp, -1.5, +1.5, -1.5, +1.5);

   //  Back to model view
   glMatrixMode(GL_MODELVIEW);
}

//
//  Draw the window
//
void Triangle::paintGL()
{
    //  Wall time
    float t = 0.001*time.elapsed();
    //  Clear screen and Z-buffer
    glClear(GL_COLOR_BUFFER_BIT | GL_DEPTH_BUFFER_BIT);

    //  Reset transformations
    glLoadIdentity();

    //  Draw RGB triangle
    glBegin(GL_TRIANGLES);
    glColor3f(1,0,0); glVertex3d( 0, 1,0);
    glColor3f(0,1,0); glVertex3d(-1,-1,0);
    glColor3f(0,0,1); glVertex3d(+1,-1,0);
    glEnd();

    //  Set rotation
    glRotated(theta , 1,1,0);

    //  Draw textured triangle
    if (mode==1)
    {
       glBindTexture(GL_TEXTURE_2D,darth);
       glEnable(GL_TEXTURE_2D);
       glColor3f(1,1,1);
       glBegin(GL_TRIANGLES);
       glTexCoord2d(0.5,1.0); glVertex3d( 0, 1,1);
       glTexCoord2d(0.0,0.0); glVertex3d(-1,-1,1);
       glTexCoord2d(1.0,0.0); glVertex3d(+1,-1,1);
       glEnd();
       glDisable(GL_TEXTURE_2D);
    }
    //  Apply shader
    else if (mode==2)
    {
       shader.bind();
       int loc = shader.uniformLocation("time");
       if (loc>=0) shader.setUniformValue(loc,t);
       glBegin(GL_TRIANGLES);
       glVertex3d( 0, 1,1);
       glVertex3d(-1,-1,1);
       glVertex3d(+1,-1,1);
       glEnd();
       shader.release();
    }
    //  Draw CMY triangle
    else
    {
       glBegin(GL_TRIANGLES);
       glColor3f(1,1,0); glVertex3d( 0, 1,1);
       glColor3f(0,1,1); glVertex3d(-1,-1,1);
       glColor3f(1,0,1); glVertex3d(+1,-1,1);
       glEnd();
    }

   //  Emit message to display
   emit message("Display at angle "+QString::number(theta)+" degrees");
}

//
//  Throw a fatal error and die
//
void Triangle::Fatal(QString message)
{
   QMessageBox::critical(this,"Triangle",message);
   exit(1);
}
